/**
 * @module RestoreHelpers
 */
import { WorkspaceWindow, WorkspaceApp } from './workspaces';
/**
 * A simple restore handler that can be passed to {@link setRestoreHandler} when no custom logic is needed.
 *
 * This restore handler will open any child windows which are not currently running,
 * and move new and existing child windows to their expected positions for the workspace being restored.
 *
 * ``` ts
 * import {workspaces, restoreHelpers} from 'openfin-layouts';
 *
 * workspaces.setRestoreHandler(restoreHelpers.standardRestoreHandler);
 * ```
 *
 * @param workspaceApp Describes the running app's child windows and their expected post-restore state.
 * See {@link WorkspaceApp} and {@link setRestoreHandler} for more information on what this will contain.
 */
export declare function standardRestoreHandler(workspaceApp: WorkspaceApp): Promise<WorkspaceApp>;
/**
 * An additional helper function that can be used to create custom restore handlers.
 *
 * Given a {@link WorkspaceWindow} it will determine if the window is open and then
 * call {@link createChild} or {@link positionChild} as appropriate.
 *
 * @param workspaceWindow Object containing details of the window identity and how it should
 * be positioned/created.
 */
export declare function createOrPositionChild(workspaceWindow: WorkspaceWindow): Promise<void>;
/**
 * Given a {@link WorkspaceWindow} for a non-open window, this function will create the window, position it,
 * and update its visibility, state, and frame to fit with the restoring workspace.
 *
 * Called by the {@link standardRestoreHandler} for any non-open windows in the WorkspaceApp.
 *
 * @param workspaceWindow Object containing details of the window identity and how it should
 * be created.
 *
 * @throws `Error`: If there is an existing child window with the same name.
 */
export declare function createChild(workspaceWindow: WorkspaceWindow): Promise<void>;
/**
 * Given a {@link WorkspaceWindow} object for a currently open window, this will position and
 * update the visibility, state and frame of that window to fit with the restoring
 * workspace.
 *
 * Called by the {@link standardRestoreHandler} for any currently running windows in the workspace.
 *
 * @param workspaceWindow Object containing details of the window identity and how it should
 * be positioned.
 *
 * @throws `Error`: If the window described by `workspaceWindow` is not currently running.
 */
export declare function positionChild(workspaceWindow: WorkspaceWindow): Promise<void>;
