/**
 * @module Tabbing
 */
import { Identity } from 'hadouken-js-adapter';
import { WindowIdentity } from './main';
/**
 * Event fired whenever the active tab within a tab group is changed.  See {@link addEventListener}.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 * import {TabActivatedEvent} from 'openfin-layouts/dist/client/tabbing';
 *
 * tabbing.addEventListener('tab-activated', (event: TabActivatedEvent) => {
 *     const activeTab = event.identity;
 *     console.log("Active tab:", activeTab.uuid, activeTab.name);
 * });
 * ```
 *
 * @event
 */
export interface TabActivatedEvent {
    type: 'tab-activated';
    /**
     * Identity object that uniquely identifies the current tabset.
     */
    tabstripIdentity: WindowIdentity;
    /**
     * Identifies the window that is the source of the current event.
     */
    identity: WindowIdentity;
}
/**
 * Event fired whenever the current window is removed from it's previous tabset.  See {@link addEventListener}.
 *
 * To find out which other windows are in the tabset, use the `getTabs()` method.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 * import {TabRemovedEvent} from 'openfin-layouts/dist/client/tabbing';
 *
 * tabbing.addEventListener('tab-removed', async (event: TabRemovedEvent) => {
 *     console.log("Window removed from tab group");
 * });
 * ```
 *
 * If a window is moved from one tab group to another, this will be messaged as a `tab-removed` event, followed by a `tab-added`.
 *
 * @event
 */
export interface TabRemovedEvent {
    type: 'tab-removed';
    /**
     * Identity object that uniquely identifies the current tabset.
     */
    tabstripIdentity: WindowIdentity;
    /**
     * Identifies the window that is the source of the current event.
     */
    identity: WindowIdentity;
}
/**
 * Event fired whenever the current window is tabbed. This event is used when adding windows to both new and existing
 * tabsets.  See {@link addEventListener}.
 *
 * To find out which other windows are in the tabset, use the `getTabs()` method.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 * import {TabAddedEvent} from 'openfin-layouts/dist/client/tabbing';
 *
 * tabbing.addEventListener('tab-added', async (event: TabAddedEvent) => {
 *     console.log("Window added to tab group: ", event.identity);
 *     console.log("Windows in current group: ", await tabbing.getTabs());
 * });
 * ```
 *
 * If a window is moved from one tab group to another, this will be messaged as a `tab-removed` event, followed by a `tab-added`.
 *
 * @event
 */
export interface TabAddedEvent {
    type: 'tab-added';
    /**
     * Identity object that uniquely identifies the current tabset.
     */
    tabstripIdentity: WindowIdentity;
    /**
     * Identifies the window that is the source of the current event.
     */
    identity: WindowIdentity;
    /**
     * The properties of the newly-added tab.
     *
     * These will be generated from the `identity` window, or will be whatever properties were previously set for the `identity` window using
     * {@link updateTabProperties}.
     */
    properties: TabProperties;
    /**
     * The index at which the tab was inserted.
     *
     * An integer in the range `[0, <tab count>-1]`.
     */
    index: number;
}
/**
 * Event fired whenever a windows tab properties are {@link updateTabProperties|updated}.  See {@link addEventListener}.
 *
 * The event will always contain the full properties of the tab, even if only a subset of them were updated.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 * import {TabPropertiesUpdatedEvent} from 'openfin-layouts/dist/client/tabbing';
 *
 * tabbing.addEventListener('tab-properties-updated', (event: TabPropertiesUpdatedEvent) => {
 *     const tabIdentity = event.identity;
 *     const properties = event.properties;
 *     console.log(`Properties for ${tabIdentity.uuid}/${tabIdentity.name} are:`, properties);
 * });
 * ```
 *
 * @event
 */
export interface TabPropertiesUpdatedEvent {
    type: 'tab-properties-updated';
    /**
     * Identifies the window that is the source of the current event.
     */
    identity: WindowIdentity;
    /**
     * New tab properties.
     *
     * This will always contain the full set of properties for the tab, even if only a subset of the properties were
     * updated in the {@link updateTabProperties} call.
     */
    properties: TabProperties;
}
/**
 * @hidden
 */
export declare type TabbingEvent = TabAddedEvent | TabRemovedEvent | TabActivatedEvent | TabPropertiesUpdatedEvent;
/**
 * Represents the state of a tab within a tabstrip.
 *
 * These properties will be passed to the tabstrip whenever a tab is added. Tabstrips can also update these properties
 * at any time, and the service will persist these changes (See {@link updateTabProperties}).
 */
export interface TabProperties {
    /**
     * Tab title - the text that is shown on the tab widget so that a user can identify the contents of that tab.
     *
     * This will be initialised to the title of the page, or the 'name' of the associated window object if no title is defined.
     */
    title: string;
    /**
     * URL to an icon image that will be displayed within the tab widget.
     */
    icon: string;
}
/**
 * Configuration options that can be set on a per-application basis, to control the tabbing behavior of any windows
 * belonging to that application.
 *
 * These parameters are set via the {@link setTabstrip} API.
 */
export interface ApplicationUIConfig {
    /**
     * The URL of the tabstrip to use for any tab groups created by this application.
     */
    url: string;
    /**
     * The height of the tabstrip window referenced by 'url', in pixels.
     */
    height: number;
}
export declare function addEventListener(eventType: 'tab-added', listener: (event: TabAddedEvent) => void): void;
export declare function addEventListener(eventType: 'tab-removed', listener: (event: TabRemovedEvent) => void): void;
export declare function addEventListener(eventType: 'tab-activated', listener: (event: TabActivatedEvent) => void): void;
export declare function addEventListener(eventType: 'tab-properties-updated', listener: (event: TabPropertiesUpdatedEvent) => void): void;
export declare function removeEventListener(eventType: 'tab-added', listener: (event: TabAddedEvent) => void): void;
export declare function removeEventListener(eventType: 'tab-removed', listener: (event: TabRemovedEvent) => void): void;
export declare function removeEventListener(eventType: 'tab-activated', listener: (event: TabActivatedEvent) => void): void;
export declare function removeEventListener(eventType: 'tab-properties-updated', listener: (event: TabPropertiesUpdatedEvent) => void): void;
/**
 * Returns array of window identity references for tabs belonging to the tab group of the provided window context.
 *
 * If there is no tab group associated with the window context, will resolve to null.
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Gets all tabs for the current window context.
 * tabbing.getTabs();
 *
 * // Get all tabs for another window context.
 * tabbing.getTabs({uuid: "sample-window-uuid", name: "sample-window-name"});
 * ```
 *
 * @param identity The window context, defaults to the current window.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 */
export declare function getTabs(identity?: Identity): Promise<WindowIdentity[] | null>;
/**
 * Creates the custom tabstrip + configuration for the entire application.  An application cannot have different windows using different tabstrip UIs.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * tabbing.setTabstrip({url: 'https://localhost/customTabstrip.html', height: 60});
 * ```
 *
 * @param config The {@link ApplicationUIConfig| Application UI Configuration} object.
 * @throws `Error`: If `config` is not a valid {@link ApplicationUIConfig}
 * @throws `Error`: If `config.url` is not a valid URL/URI.
 */
export declare function setTabstrip(config: ApplicationUIConfig): Promise<void>;
/**
 * Creates a tabgroup with the provided windows.  The first window in the set will be used to define the tab strips properties.  See {@link setTabstrip}.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * tabbing.createTabGroup([{uuid: "App1", name: "App1"}, {uuid: "App2", name: "App2"}, {uuid: "App3", name: "App3"}]);
 * ```
 *
 * @param identities Array of window {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identities} which will be added to the
 * new tab group.
 * @param activeTab The {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity} of the window to set as the active tab in
 * the group.  If not provided, the first tab in the tab group will be set as the active tab.
 * @throws `Error`: If one of the provided {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identities} is not valid.
 * @throws `Error`: If duplicate {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identities} are provided.
 * @throws `Error`: If the provided value is not an array or less than 2 windows identities were provided.
 */
export declare function createTabGroup(identities: Identity[], activeTab?: Identity): Promise<void>;
/**
 * Tabs two windows together.  If the targetWindow is already in a group, the tab will be added to that group.
 *
 * The added tab will be brought into focus.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Tab App1 to App2
 * tabbing.tabWindowToWindow({uuid: 'App1', name: 'App1'}, {uuid: 'App2', name: 'App2'});
 * ```
 *
 * @param windowToAdd The identity of the window to add to the tab group.
 * @param targetWindow The identity of the window to create a tab group on.
 * @throws `Error`: If the {@link ApplicationUIConfig| App Config} does not match between the target and window to add.
 * @throws `Error`: If the `targetWindow` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If the `windowToAdd` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 */
export declare function tabWindowToWindow(windowToAdd: Identity, targetWindow: Identity): Promise<void>;
/**
 * Removes the specified window context from its tab group.  This does not close the window.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Remove the window from its tab group.
 * tabbing.removeTab();
 *
 * // Remove another window from its tab group.
 * tabbing.removeTab({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to remove.  If no `Identity` is provided as an argument, the current window context will be used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` refers to a window that doesn't exist, or is deregistered from the service.
 */
export declare function removeTab(identity?: Identity): Promise<void>;
/**
 * Sets the window context as the active tab in its tab group.  Active tabs are brought to the front of the tab group and shown.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts'
 *
 * // Sets the current window as active in the tab group.
 * tabbing.setActiveTab()
 *
 * // Sets another window context as the active tab.
 * tabbing.setActiveTab({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to set as active.  If no `Identity` is provided as an argument the current window context will be used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` is not an existing tab in a tabstrip.
 */
export declare function setActiveTab(identity?: Identity): Promise<void>;
/**
 * Closes the tab for the window context and removes it from its associated tab group.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Closes the current window context tab.
 * tabbing.closeTab();
 *
 * // Closes another windows context tab.
 * tabbing.closeTab({uuid: 'App1', name: 'App1'});
 * ```
 *
 * Note that the `identity` doesn't need to be tabbed for this call to take effect - so long as the window is known with the service.
 *
 * @param identity Identity of the window context to close.  If no `Identity` is provided as an argument the current window context will be used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` refers to a window that doesn't exist, or is deregistered from the service.
 */
export declare function closeTab(identity?: Identity): Promise<void>;
/**
 * Minimizes the tab group for the window context.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Minimizes the tab group for the current window context.
 * tabbing.minimizeTabGroup();
 *
 * // Minimizes the tab group for another windows context.
 * tabbing.minimizeTabGroup({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to minimize the tab group for.  If no `Identity` is provided as an argument the current window context will be
 * used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` is not an existing tab in a tabstrip.
 */
export declare function minimizeTabGroup(identity?: Identity): Promise<void>;
/**
 * Maximizes the tab group for the window context.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Maximizes the tab group for the current window context.
 * tabbing.maxmimizeTabGroup();
 *
 * // Maximizes the tab group for another windows context.
 * tabbing.maximizeTabGroup({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to maximize the tab group for.  If no `Identity` is provided as an argument the current window context will be
 * used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` is not an existing tab in a tabstrip.
 */
export declare function maximizeTabGroup(identity?: Identity): Promise<void>;
/**
 * Restores the tab group for the window context to its normal state.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Restores the tab group for the current window context.
 * tabbing.restoreTabGroup();
 *
 * // Restores the tab group for another windows context.
 * tabbing.restoreTabGroup({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to restore the tab group for.  If no `Identity` is provided as an argument the current window context will be
 * used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` is not an existing tab in a tabstrip.
 */
export declare function restoreTabGroup(identity?: Identity): Promise<void>;
/**
 * Closes the tab group for the window context.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Closes the tab group for the current window context.
 * tabbing.closeTabGroup();
 *
 * // Closes the tab group for another windows context.
 * tabbing.closeTabGroup({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param identity Identity of the window context to close the tab group for.  If no `Identity` is provided as an argument the current window context will be
 * used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If `identity` is not an existing tab in a tabstrip.
 */
export declare function closeTabGroup(identity?: Identity): Promise<void>;
/**
 * Updates a tab's properties. Properties for a tab include its title and icon when in a tab group.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Updating only some properties for the current window context.
 * tabbing.updateTabProperties({title: 'An Awesome Tab!'});
 *
 * // Update all properties for the current window context.
 * tabbing.updateTabProperties({title: 'An Awesome Tab!', icon: 'http://openfin.co/favicon.ico'});
 *
 * // Update properties for another windows context.
 * tabbing.updateTabProperties({title: 'An Awesome Tab'}, {uuid: 'App1', name: 'App1'});
 * ```
 * @param properties Properties object for the tab to consume.
 * @param identity Identity of the window context set the properties on.  If no `Identity` is provided as an argument the current window context will be used.
 * @throws `Error`: If `identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 */
export declare function updateTabProperties(properties: Partial<TabProperties>, identity?: Identity): Promise<void>;
/**
 * Adds the provided window context as a tab to the current window context.
 *
 * The added tab will be brought into focus.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Tab App2 to current window.
 * tabbing.tabToSelf({uuid: 'App2', name: 'App2'});
 * ```
 *
 * @param Identity The identity of the window to add as a tab.
 * @throws `Error`: If the {@link ApplicationUIConfig| App Config} does not match between the window to add and the current window context.
 * @throws `Error`: If the `Identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If the `Identity` matches the calling windows `Identity`.
 */
export declare function tabToSelf(identity: Identity): Promise<void>;
/**
 * Adds the current window context as a tab to the provided window context.
 *
 * The added tab will be brought into focus.
 *
 * ```ts
 * import {tabbing} from 'openfin-layouts';
 *
 * // Tab current window to App1.
 * tabbing.tabSelfTo({uuid: 'App1', name: 'App1'});
 * ```
 *
 * @param Identity The identity of the window to add the current window context as a tab to.
 * @throws `Error`: If the {@link ApplicationUIConfig| App Config} does not match between the window to add and the current window context.
 * @throws `Error`: If the `Identity` is not a valid {@link https://developer.openfin.co/docs/javascript/stable/global.html#Identity | Identity}.
 * @throws `Error`: If the `Identity` matches the calling windows `Identity`.
 */
export declare function tabSelfTo(identity: Identity): Promise<void>;
